﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;

namespace ZerimThing
{
    internal partial class frmProperties : Form
    {
        private List<clsProperty> NewProperties;

        internal frmProperties(List<clsProperty> iOldProperties)
        {
            this.InitializeComponent();

            this.NewProperties = iOldProperties;

            this.UpdateGUI();

        }

        internal void UpdateGUI()
        {
            this.lvProperties.Items.Clear();

            foreach(clsProperty tProperty in this.NewProperties)
            {
                ListViewItem tPropertyItem = new ListViewItem();

                tPropertyItem.Text = tProperty.Name;
                tPropertyItem.Tag = tProperty;

                this.lvProperties.Items.Add(tPropertyItem);
            }
        }

        private void btnAddProperty_Click(object sender, EventArgs e)
        {
            frmAddProperty tAddPropertyWindow = new frmAddProperty(frmMain.AllProperties);
            DialogResult tDialogResult = tAddPropertyWindow.ShowDialog();

            if(tDialogResult != DialogResult.OK)
            {
                return;
            }

            clsProperty tNewProperty = tAddPropertyWindow.GetNewProperty();

            // check that this property is not already added
            foreach(clsProperty tProperty in this.NewProperties)
            {
                if(tProperty.Name == tNewProperty.Name)
                {
                    // this is hokey for the user, two things you could do:
                    // - warn them in the other dialog (meh)
                    // - automatically update the existing property (great)
                    MessageBox.Show("Can't add property: Property is already added.");
                    return;
                }
            }

            this.NewProperties.Add(tNewProperty);

            UpdateGUI();
        }

        private void btnDeleteProperty_Click(object sender, EventArgs e)
        {
            if(lvProperties.SelectedItems.Count != 1)
            {
                // nothing selected (deselecting everything counts as a "change")
                return;
            }

            // since we only allow deleting one property at a time, take the first selected item
            ListViewItem tSelectedItem = lvProperties.SelectedItems[0];

            //--------------------------------------------------------------------------------------------------------
            // safety first! The tag could contain garbage data. I'm kinda paranoid about this though...

            bool tTagDataValid = false;

            // check to make sure it actually has any data at all
            if(tSelectedItem.Tag != null)
            {
                // if it does, check if it's the right type
                if(tSelectedItem.Tag is clsProperty)
                {
                    tTagDataValid = true;
                }
            }

            if(!tTagDataValid)
            {
                // this shouldn't happen so let's assert so the program will stop for us so we can find the problem.
                Debug.Fail("Listview item tag data invalid");

                // for release data, if there's no data or invalid data, let's stop now, it's better to stop than it is to
                // try to display garbage.
                return;
            }

            //-------------------------------------------------------------------------------------------------------
            // tag data is valid, let's find out which one to delete

            clsProperty tSelectedProperty = (clsProperty) tSelectedItem.Tag;

            // note: this could be sped up using Dictionaries
            foreach(clsProperty tProperty in this.NewProperties)
            {
                if(tProperty.Name == tSelectedProperty.Name)
                {
                    this.NewProperties.Remove(tSelectedProperty);
                    // we found it: break out of the foreach loop
                    break;
                }
            }

            this.UpdateGUI();
        }
        

        private void btnOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }

        internal List<clsProperty> GetNewPropertiesList()
        {
            return this.NewProperties;
        }   

     
    }
}
